@@ -412,7 +412,7 @@ static void _stp_pmap_del(PMAP pmap)
 }
 
 /* sort keynum values */
-#define SORT_COUNT -5
+#define SORT_COUNT -5 /* see also translate.cxx:visit_foreach_loop */
 #define SORT_SUM   -4
 #define SORT_MIN   -3
 #define SORT_MAX   -2
@@ -719,7 +719,7 @@ static MAP _stp_pmap_agg (PMAP pmap)
 {
 	int i, hash;
 	MAP m, agg;
-	struct map_node *ptr, *aptr;
+	struct map_node *ptr, *aptr = NULL;
 	struct hlist_head *head, *ahead;
 	struct hlist_node *e, *f;
 
