@@ -2,9 +2,8 @@
 #include <linux/jhash.h>
 #include <linux/spinlock.h>
 
-// When handling memcpy() syscall tracing to notice memory map
-// changes, we need to cache memcpy() entry parameter values for
-// processing at memcpy() exit.
+#include <linux/fs.h>
+#include <linux/dcache.h>
 
 // __stp_tf_vma_lock protects the hash table.
 // Documentation/spinlocks.txt suggest we can be a bit more clever
@@ -13,24 +12,29 @@
 // contents in interrupt context (which should only ever call 
 // stap_find_vma_map_info for getting stored vma info). So we might
 // want to look into that if this seems a bottleneck.
+#ifdef CONFIG_PREEMPT_RT
+static DEFINE_RAW_RWLOCK(__stp_tf_vma_lock);
+#else
 static DEFINE_RWLOCK(__stp_tf_vma_lock);
+#endif
 
 #define __STP_TF_HASH_BITS 4
 #define __STP_TF_TABLE_SIZE (1 << __STP_TF_HASH_BITS)
 
+// Somewhat arbitrary default, this is often way too much for tracking
+// single process, but often too little when tracking whole system.
+// FIXME Would be nice to make this dynamic. PR11671
 #ifndef TASK_FINDER_VMA_ENTRY_ITEMS
-#define TASK_FINDER_VMA_ENTRY_ITEMS 100
+#define TASK_FINDER_VMA_ENTRY_ITEMS 1536
 #endif
 
 struct __stp_tf_vma_entry {
 	struct hlist_node hlist;
 
 	pid_t pid;
-	unsigned long addr;
 	unsigned long vm_start;
 	unsigned long vm_end;
-	unsigned long vm_pgoff;
-	// Is that enough?  Should we store a dcookie for vm_file?
+	const char *name;
 
 	// User data (possibly stp_module)
 	void *user;
@@ -41,14 +45,13 @@ __stp_tf_vma_free_list_items[TASK_FINDER_VMA_ENTRY_ITEMS];
 
 static struct hlist_head __stp_tf_vma_free_list[1];
 
-static struct hlist_head __stp_tf_vma_table[__STP_TF_TABLE_SIZE];
-
 static struct hlist_head __stp_tf_vma_map[__STP_TF_TABLE_SIZE];
 
-// __stp_tf_vma_initialize():  Initialize the free list.  Grabs the
-// spinlock.
+// stap_initialize_vma_map():  Initialize the free list.  Grabs the
+// spinlock.  Should be called before any of the other stap_*_vma_map
+// functions.
 static void
-__stp_tf_vma_initialize(void)
+stap_initialize_vma_map(void)
 {
 	int i;
 	struct hlist_head *head = &__stp_tf_vma_free_list[0];
@@ -93,106 +96,6 @@ __stp_tf_vma_put_free_entry(struct __stp_tf_vma_entry *entry)
 	hlist_add_head(&entry->hlist, head);
 }
 
-
-// __stp_tf_vma_hash(): Compute the vma hash.
-static inline u32
-__stp_tf_vma_hash(struct task_struct *tsk, unsigned long addr)
-{
-#ifdef CONFIG_64BIT
-    return (jhash_3words(tsk->pid, (u32)addr, (u32)(addr >> 32), 0)
-	    & (__STP_TF_TABLE_SIZE - 1));
-#else
-    return (jhash_2words(tsk->pid, addr, 0) & (__STP_TF_TABLE_SIZE - 1));
-#endif
-}
-
-
-// Get vma_entry if the vma is present in the vma hash table.
-// Returns NULL if not present. Takes a read lock on __stp_tf_vma_lock.
-static struct __stp_tf_vma_entry *
-__stp_tf_get_vma_entry(struct task_struct *tsk, unsigned long addr)
-{
-	struct hlist_head *head;
-	struct hlist_node *node;
-	struct __stp_tf_vma_entry *entry;
-
-	unsigned long flags;
-	read_lock_irqsave(&__stp_tf_vma_lock, flags);
-	head = &__stp_tf_vma_table[__stp_tf_vma_hash(tsk, addr)];
-	hlist_for_each_entry(entry, node, head, hlist) {
-		if (tsk->pid == entry->pid
-		    && addr == entry->addr) {
-			read_unlock_irqrestore(&__stp_tf_vma_lock, flags);
-			return entry;
-		}
-	}
-	read_unlock_irqrestore(&__stp_tf_vma_lock, flags);
-	return NULL;
-}
-
-// Add the vma info to the vma hash table.
-// Takes a write lock on __stp_tf_vma_lock.
-static int
-__stp_tf_add_vma(struct task_struct *tsk, unsigned long addr,
-		 struct vm_area_struct *vma)
-{
-	struct hlist_head *head;
-	struct hlist_node *node;
-	struct __stp_tf_vma_entry *entry;
-
-	unsigned long flags;
-	write_lock_irqsave(&__stp_tf_vma_lock, flags);
-	head = &__stp_tf_vma_table[__stp_tf_vma_hash(tsk, addr)];
-	hlist_for_each_entry(entry, node, head, hlist) {
-		if (tsk->pid == entry->pid
-		    && addr == entry->addr) {
-#ifdef DEBUG_TASK_FINDER_VMA
-                  printk(KERN_NOTICE
-                         "vma (pid: %d, vm_start: 0x%lx) present?\n",
-                         tsk->pid, vma->vm_start);
-#endif
-                  write_unlock_irqrestore(&__stp_tf_vma_lock, flags);
-                  return -EBUSY;	/* Already there */
-		}
-	}
-
-	// Get an element from the free list.
-	entry = __stp_tf_vma_get_free_entry();
-	if (!entry) {
-		write_unlock_irqrestore(&__stp_tf_vma_lock, flags);
-		return -ENOMEM;
-	}
-	entry->pid = tsk->pid;
-	entry->addr = addr;
-	entry->vm_start = vma->vm_start;
-	entry->vm_end = vma->vm_end;
-	entry->vm_pgoff = vma->vm_pgoff;
-	hlist_add_head(&entry->hlist, head);
-	write_unlock_irqrestore(&__stp_tf_vma_lock, flags);
-	return 0;
-}
-
-// Remove the vma entry from the vma hash table.
-// Takes a write lock on __stp_tf_vma_lock.
-static int
-__stp_tf_remove_vma_entry(struct __stp_tf_vma_entry *entry)
-{
-	struct hlist_head *head;
-	struct hlist_node *node;
-	int found = 0;
-
-	if (entry != NULL) {
-		unsigned long flags;
-		write_lock_irqsave(&__stp_tf_vma_lock, flags);
-		hlist_del(&entry->hlist);
-		__stp_tf_vma_put_free_entry(entry);
-		write_unlock_irqrestore(&__stp_tf_vma_lock, flags);
-	}
-	return 0;
-}
-
-
-
 // __stp_tf_vma_map_hash(): Compute the vma map hash.
 static inline u32
 __stp_tf_vma_map_hash(struct task_struct *tsk)
@@ -214,7 +117,7 @@ __stp_tf_get_vma_map_entry_internal(struct task_struct *tsk,
 	head = &__stp_tf_vma_map[__stp_tf_vma_map_hash(tsk)];
 	hlist_for_each_entry(entry, node, head, hlist) {
 		if (tsk->pid == entry->pid
-		    && vm_start == entry->addr) {
+		    && vm_start == entry->vm_start) {
 			return entry;
 		}
 	}
@@ -223,10 +126,11 @@ __stp_tf_get_vma_map_entry_internal(struct task_struct *tsk,
 
 
 // Add the vma info to the vma map hash table.
+// Caller is responsible for name lifetime.
 static int
-stap_add_vma_map_info(struct task_struct *tsk, unsigned long vm_start,
-		      unsigned long vm_end, unsigned long vm_pgoff,
-		      void *user)
+stap_add_vma_map_info(struct task_struct *tsk,
+		      unsigned long vm_start, unsigned long vm_end,
+		      const char *name, void *user)
 {
 	struct hlist_head *head;
 	struct hlist_node *node;
@@ -256,10 +160,9 @@ stap_add_vma_map_info(struct task_struct *tsk, unsigned long vm_start,
 
 	// Fill in the info
 	entry->pid = tsk->pid;
-	//entry->addr = addr; ???
 	entry->vm_start = vm_start;
 	entry->vm_end = vm_end;
-	entry->vm_pgoff = vm_pgoff;
+	entry->name = name;
 	entry->user = user;
 
 	head = &__stp_tf_vma_map[__stp_tf_vma_map_hash(tsk)];
@@ -272,8 +175,7 @@ stap_add_vma_map_info(struct task_struct *tsk, unsigned long vm_start,
 // Remove the vma entry from the vma hash table.
 // Returns -ESRCH if the entry isn't present.
 static int
-stap_remove_vma_map_info(struct task_struct *tsk, unsigned long vm_start,
-			     unsigned long vm_end, unsigned long vm_pgoff)
+stap_remove_vma_map_info(struct task_struct *tsk, unsigned long vm_start)
 {
 	struct hlist_head *head;
 	struct hlist_node *node;
@@ -299,9 +201,9 @@ stap_remove_vma_map_info(struct task_struct *tsk, unsigned long vm_start,
 // Returns -ESRCH if not present.  The __stp_tf_vma_lock must *not* be
 // locked before calling this function.
 static int
-stap_find_vma_map_info(struct task_struct *tsk, unsigned long vm_addr,
+stap_find_vma_map_info(struct task_struct *tsk, unsigned long addr,
 		       unsigned long *vm_start, unsigned long *vm_end,
-		       unsigned long *vm_pgoff, void **user)
+		       const char **name, void **user)
 {
 	struct hlist_head *head;
 	struct hlist_node *node;
@@ -314,8 +216,8 @@ stap_find_vma_map_info(struct task_struct *tsk, unsigned long vm_addr,
 	head = &__stp_tf_vma_map[__stp_tf_vma_map_hash(tsk)];
 	hlist_for_each_entry(entry, node, head, hlist) {
 		if (tsk->pid == entry->pid
-		    && vm_addr >= entry->vm_start
-		    && vm_addr < entry->vm_end) {
+		    && addr >= entry->vm_start
+		    && addr < entry->vm_end) {
 			found_entry = entry;
 			break;
 		}
@@ -325,8 +227,8 @@ stap_find_vma_map_info(struct task_struct *tsk, unsigned long vm_addr,
 			*vm_start = found_entry->vm_start;
 		if (vm_end != NULL)
 			*vm_end = found_entry->vm_end;
-		if (vm_pgoff != NULL)
-			*vm_pgoff = found_entry->vm_pgoff;
+		if (name != NULL)
+			*name = found_entry->name;
 		if (user != NULL)
 			*user = found_entry->user;
 		rc = 0;
@@ -342,7 +244,7 @@ stap_find_vma_map_info(struct task_struct *tsk, unsigned long vm_addr,
 static int
 stap_find_vma_map_info_user(struct task_struct *tsk, void *user,
 			    unsigned long *vm_start, unsigned long *vm_end,
-			    unsigned long *vm_pgoff)
+			    const char **name)
 {
 	struct hlist_head *head;
 	struct hlist_node *node;
@@ -365,8 +267,8 @@ stap_find_vma_map_info_user(struct task_struct *tsk, void *user,
 			*vm_start = found_entry->vm_start;
 		if (vm_end != NULL)
 			*vm_end = found_entry->vm_end;
-		if (vm_pgoff != NULL)
-			*vm_pgoff = found_entry->vm_pgoff;
+		if (name != NULL)
+			*name = found_entry->name;
 		rc = 0;
 	}
 	read_unlock_irqrestore(&__stp_tf_vma_lock, flags);
