@@ -76,8 +76,12 @@ struct unwind_frame_info
 
 
 static inline void arch_unw_init_frame_info(struct unwind_frame_info *info,
-                                            /*const*/ struct pt_regs *regs)
+                                            /*const*/ struct pt_regs *regs,
+					    int sanitize)
 {
+	if (sanitize) /* We are only prepared to use full reg sets. */
+		_stp_error("Impossible to sanitize i386 pr_regs");
+
 	if (user_mode_vm(regs))
 		info->regs = *regs;
 	else {
