@@ -64,7 +64,7 @@ static int stap_uprobe_change_plus (struct task_struct *tsk, unsigned long reloc
     }
     mutex_unlock (& stap_uprobes_lock);
     #ifdef DEBUG_UPROBES
-    _stp_dbug(__FUNCTION__,__LINE__, "+uprobe spec %d idx %d process %s[%d] addr %p pp %s\n", spec_index, (slotted_p ? i : -1), tsk->comm, tsk->tgid, (void*)(relocation+sups->address), sups->pp);
+    _stp_dbug(__FUNCTION__,__LINE__, "+uprobe spec %d idx %d process %s[%d] addr %p pp %s\n", spec_index, (slotted_p ? i : -1), tsk->comm, tsk->tgid, (void*)(relocation+sups->address), sups->probe.pp);
     #endif
 
     /* Here, slotted_p implies that `i' points to the single
@@ -97,7 +97,7 @@ static int stap_uprobe_change_plus (struct task_struct *tsk, unsigned long reloc
         rc = register_uprobe (& sup->up);
       }
       if (rc) { /* failed to register */
-        _stp_warn ("u*probe failed %s[%d] '%s' addr %p rc %d\n", tsk->comm, tsk->tgid, sups->pp, (void*)(relocation + sups->address), rc);
+        _stp_warn ("u*probe failed %s[%d] '%s' addr %p rc %d\n", tsk->comm, tsk->tgid, sups->probe.pp, (void*)(relocation + sups->address), rc);
 	/* NB: we need to release this slot,
 	   so we need to borrow the mutex temporarily. */
         mutex_lock (& stap_uprobes_lock);
@@ -202,7 +202,7 @@ static int stap_uprobe_change_minus (struct task_struct *tsk, unsigned long relo
     if (sups->return_p && sup->urp.u.pid == tsk->tgid && sup->urp.u.vaddr >= relocation && sup->urp.u.vaddr < relocation+length) { /* in range */
       
       #ifdef DEBUG_UPROBES
-      _stp_dbug (__FUNCTION__,__LINE__, "-uretprobe spec %d idx %d process %s[%d] addr %p pp %s\n", sup->spec_index, i, tsk->comm, tsk->tgid, (void*) sup->urp.u.vaddr, sups->pp);
+      _stp_dbug (__FUNCTION__,__LINE__, "-uretprobe spec %d idx %d process %s[%d] addr %p pp %s\n", sup->spec_index, i, tsk->comm, tsk->tgid, (void*) sup->urp.u.vaddr, sups->probe.pp);
       #endif
       #if (UPROBES_API_VERSION >= 2)
       unmap_uretprobe (& sup->urp);
@@ -217,7 +217,7 @@ static int stap_uprobe_change_minus (struct task_struct *tsk, unsigned long relo
     } else if (!sups->return_p && sup->up.pid == tsk->tgid && sup->up.vaddr >= relocation && sup->up.vaddr < relocation+length) { /* in range */
       
       #ifdef DEBUG_UPROBES
-      _stp_dbug (__FUNCTION__,__LINE__, "-uprobe spec %d idx %d process %s[%d] reloc %p pp %s\n", sup->spec_index, i, tsk->comm, tsk->tgid, (void*) sup->up.vaddr, sups->pp);
+      _stp_dbug (__FUNCTION__,__LINE__, "-uprobe spec %d idx %d process %s[%d] reloc %p pp %s\n", sup->spec_index, i, tsk->comm, tsk->tgid, (void*) sup->up.vaddr, sups->probe.pp);
       #endif
       #if (UPROBES_API_VERSION >= 2)
       unmap_uprobe (& sup->up);
@@ -255,7 +255,7 @@ static int stap_uprobe_process_found (struct stap_task_finder_target *tgt, struc
 }
 
 /* The task_finder_mmap_callback */
-static int stap_uprobe_mmap_found (struct stap_task_finder_target *tgt, struct task_struct *tsk, char *path, unsigned long addr, unsigned long length, unsigned long offset, unsigned long vm_flags) {
+static int stap_uprobe_mmap_found (struct stap_task_finder_target *tgt, struct task_struct *tsk, char *path, struct dentry *dentry, unsigned long addr, unsigned long length, unsigned long offset, unsigned long vm_flags) {
   const struct stap_uprobe_tf *stf = container_of(tgt, struct stap_uprobe_tf, finder);
   /* 1 - shared libraries' executable segments load from offset 0
        - ld.so convention offset != 0 is now allowed
