@@ -122,8 +122,7 @@ be_derived_probe_group::emit_module_decls (systemtap_session& s)
   sort(probes.begin(), probes.end(), be_derived_probe::comp);
 
   s.op->newline() << "static struct stap_be_probe {";
-  s.op->newline(1) << "void (*ph)(struct context*);";
-  s.op->newline() << "const char* pp;";
+  s.op->newline(1) << "struct stap_probe probe;";
   s.op->newline() << "int state, type;";
   s.op->newline(-1) << "} stap_be_probes[] = {";
   s.op->indent(1);
@@ -131,9 +130,7 @@ be_derived_probe_group::emit_module_decls (systemtap_session& s)
   for (unsigned i=0; i < probes.size(); i++)
     {
       s.op->newline () << "{";
-      s.op->line() << " .pp="
-                   << lex_cast_qstring (*probes[i]->sole_location()) << ",";
-      s.op->line() << " .ph=&" << probes[i]->name << ",";
+      s.op->line() << " .probe=" << common_probe_init (probes[i]) << ",";
       s.op->line() << " .state=" << states[probes[i]->type] << ",";
       s.op->line() << " .type=" << probes[i]->type;
       s.op->line() << " },";
@@ -142,8 +139,8 @@ be_derived_probe_group::emit_module_decls (systemtap_session& s)
 
   s.op->newline() << "static void enter_be_probe (struct stap_be_probe *stp) {";
   s.op->indent(1);
-  common_probe_entryfn_prologue (s.op, "stp->state", "stp->pp", false);
-  s.op->newline() << "(*stp->ph) (c);";
+  common_probe_entryfn_prologue (s.op, "stp->state", "stp->probe", false);
+  s.op->newline() << "(*stp->probe.ph) (c);";
   common_probe_entryfn_epilogue (s.op, false);
   s.op->newline(-1) << "}";
 }
